# Dictionaries used to map column names to valid argument names

LEAD_COLUMN_MAP = {'first_name': ['first', 'fn', 'firstname'],
                   'last_name': ['last', 'ln', 'lastname'],
                   'phone_number': ['phone', 'cell', 'phonenumber'],
                   'email': ['email_address', 'emailaddress'],
                   'follow_up': ['followup']
                   }
