
find_people_response = {
    'vanId': 19722445,
    'firstName': 'Bob',
    'lastName': 'Smith',
    'middleName': None,
    'suffix': None,
    'title': None,
    'contactMode': None,
    'organizationContactCommonName': None,
    'organizationContactOfficialName': None,
    'salutation': 'Bob',
    'formalSalutation': 'Bob Smith',
    'additionalSalutation': None,
    'preferredPronoun': None,
    'envelopeName': 'Bob Smith',
    'formalEnvelopeName': 'Bob Smith',
    'additionalEnvelopeName': None,
    'contactMethodPreferenceCode': None,
    'nickname': None,
    'website': None,
    'professionalSuffix': None,
    'party': None,
    'employer': None,
    'occupation': None,
    'sex': 'F',
    'dateOfBirth': '1975-09-18T00:00:00Z',
    'selfReportedRace': None,
    'selfReportedEthnicity': None,
    'selfReportedRaces': None,
    'selfReportedEthnicities': None,
    'selfReportedGenders': None,
    'selfReportedSexualOrientations': None,
    'selfReportedLanguagePreference': None,
    'emails': None,
    'phones': None,
    'addresses': None,
    'recordedAddresses': None,
    'identifiers': None,
    'codes': None,
    'customFields': None,
    'contributionSummary': None,
    'suppressions': None,
    'caseworkCases': None,
    'caseworkIssues': None,
    'caseworkStories': None,
    'notes': None,
    'scores': None,
    'customProperties': None,
    'electionRecords': None,
    'membershipStatus': None,
    'organizationRoles': None,
    'districts': None
}

get_person_response = {
    'vanId': 19722445,
    'firstName': 'Bob',
    'lastName': 'Smith',
    'middleName': None,
    'suffix': None,
    'title': None,
    'contactMode': None,
    'organizationContactCommonName': None,
    'organizationContactOfficialName': None,
    'salutation': 'Bob',
    'formalSalutation': 'Bob Smith',
    'additionalSalutation': None,
    'preferredPronoun': None,
    'envelopeName': 'Bob Smith',
    'formalEnvelopeName': 'Bob Smith',
    'additionalEnvelopeName': None,
    'contactMethodPreferenceCode': None,
    'nickname': None,
    'website': None,
    'professionalSuffix': None,
    'party': None,
    'employer': None,
    'occupation': None,
    'sex': 'F',
    'dateOfBirth': '1975-09-18T00:00:00Z',
    'selfReportedRace': None,
    'selfReportedEthnicity': None,
    'selfReportedRaces': None,
    'selfReportedEthnicities': None,
    'selfReportedGenders': None,
    'selfReportedSexualOrientations': None,
    'selfReportedLanguagePreference': None,
    'emails': [],
    'phones': [{
        'phoneId': 15406767,
        'phoneNumber': '4142020792',
        'ext': None,
        'dateCreated': '2019-01-11T12:19:00Z',
        'phoneType': 'Cell',
        'isPreferred': True,
        'smsOptInStatus': 'Unknown',
        'phoneOptInStatus': 'Unknown',
        'isCellStatus': {
            'statusId': 1,
            'statusName': 'Verified Cell'
        }
    }],
    'addresses': [{
        'addressId': None,
        'addressLine1': '900 N Washtenaw Ave',
        'addressLine2': None,
        'addressLine3': None,
        'city': 'Chicago',
        'stateOrProvince': 'IL',
        'zipOrPostalCode': '60622-4455',
        'geoLocation': None,
        'countryCode': 'US',
        'preview': '900 N Washtenaw Ave ',
        'type': 'Voting',
        'isPreferred': True,
        'streetAddress': '900 N Washtenaw Ave',
        'displayMode': 'Standardized'
    }, {
        'addressId': None,
        'addressLine1': '900 N Washtenaw Ave',
        'addressLine2': None,
        'addressLine3': None,
        'city': 'Chicago',
        'stateOrProvince': 'IL',
        'zipOrPostalCode': '60622-4455',
        'geoLocation': None,
        'countryCode': 'US',
        'preview': '900 N Washtenaw Ave ',
        'type': 'Mailing',
        'isPreferred': None,
        'streetAddress': '900 N Washtenaw Ave',
        'displayMode': 'Standardized'
    }],
    'recordedAddresses': [],
    'identifiers': [{
        'type': 'StateFileID',
        'externalId': 'KLYW682Z'
    }],
    'codes': None,
    'customFields': [],
    'contributionSummary': None,
    'suppressions': [],
    'caseworkCases': None,
    'caseworkIssues': None,
    'caseworkStories': None,
    'notes': None,
    'scores': None,
    'customProperties': None,
    'electionRecords': None,
    'membershipStatus': None,
    'organizationRoles': None,
    'districts': None
}
